@@ -5,7 +5,7 @@
 # not a representation of a generic WebAuth token; it is only an abstract base
 # class.
 #
-# Written by Russ Allbery <rra@stanford.edu>
+# Written by Russ Allbery <eagle@eyrie.org>
 # Copyright 2012, 2013
 #     The Board of Trustees of the Leland Stanford Junior University
 #
@@ -41,7 +41,7 @@ our $VERSION;
 # This version matches the version of WebAuth with which this module was
 # released, but with two digits for the minor and patch versions.
 BEGIN {
-    $VERSION = '4.0505';
+    $VERSION = '4.0600';
 }
 
 # Constructor.  Requires a WebAuth context and optionally can take an encoded
@@ -211,7 +211,7 @@ to create a new, empty token of that type.
 
 =head1 AUTHOR
 
-Russ Allbery <rra@stanford.edu>
+Russ Allbery <eagle@eyrie.org>
 
 =head1 SEE ALSO
 
