@@ -1,7 +1,7 @@
 # An object encapsulating a request to a WebKDC.
 #
 # Written by Roland Schemers
-# Copyright 2002, 2003, 2005, 2009, 2012, 2013
+# Copyright 2002, 2003, 2005, 2009, 2012, 2013, 2014
 #     The Board of Trustees of the Leland Stanford Junior University
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -32,7 +32,7 @@ our $VERSION;
 # This version matches the version of WebAuth with which this module was
 # released, but with two digits for the minor and patch versions.
 BEGIN {
-    $VERSION = '4.0505';
+    $VERSION = '4.0600';
 }
 
 # Create a new, empty request.
@@ -108,6 +108,7 @@ __END__
 
 =for stopwords
 WebAuth WebKDC login IP otp username decrypt WebLogin Allbery multifactor
+ADDR
 
 =head1 NAME
 
@@ -251,7 +252,7 @@ to create a valid WebKDC::WebRequest.
 
 =head1 AUTHOR
 
-Roland Schemers and Russ Allbery <rra@stanford.edu>
+Roland Schemers and Russ Allbery <eagle@eyrie.org>
 
 =head1 SEE ALSO
 
